/*******************************************************************************
 *
 * File Name: olc.cpp
 * Author: Richard Baxter
 * Project: Optional Line Creator
 * Project Version: 1b9a 12-June-07
 * Comments: this olc.cpp contains code from sprite.cpp from LRRC 2c4a (24-Feb-07)
 *				NB - no recursive parsing.
 *
 *******************************************************************************/


#include <string>
#include <iostream>
#include <fstream>
#include <time.h>
#include <math.h>

#include "olc.h"
#include "parser.h"
#include "referenceClass.h"
#include "vector.h"
#include "referenceManipulation.h"
//using namespace std;


int numpolysfound = 0;
int totalNumOptionalLines = 0;
int samespolysfound = 0;
int ordinaryedgesfound = 0;
int alreadyaddedoptionallinesfound = 0;
int edgeAndOptionalLineOnSamePlaneFound = 0;

int main()
{
	int result = true;

	//for parsing cs.txt
	if(!createPartWithOptionalLines())
	{
		result = false;
	}

	return result;
}

bool createPartWithOptionalLines()
{
	bool result = true;
	char partFileOriginalName[] = "part.dat";
	char partFileModifiedName[] = "partNew.dat";

	ifstream partFileOriginal(partFileOriginalName);

	if(!partFileOriginal.rdbuf( )->is_open( ))
	{
		cout << "part.dat does not exist in current directory." << endl;
		result = false;
	}
	else
	{
		partFileOriginal.close();

		//declare initial scene references
		Reference * initialReferenceInPart = new Reference();
		Reference * topLevelReferenceInPart = new Reference();		//not used in OLC project

		if(!parseFile(partFileOriginalName, initialReferenceInPart, topLevelReferenceInPart))
		{
			result = false;
		}

		if(!partOptionalLinesFileOutput(partFileModifiedName, partFileOriginalName, initialReferenceInPart))
		{
			result = false;
		}
	}

	cout << "numpolysfound = " << numpolysfound << endl;
	cout << "totalNumOptionalLines = " << totalNumOptionalLines << endl;
	cout << "samespolysfound = " << samespolysfound << endl;
	cout << "ordinaryedgesfound = " << ordinaryedgesfound << endl;
	cout << "alreadyaddedoptionallinesfound = " << alreadyaddedoptionallinesfound << endl;
	cout << "edgeAndOptionalLineOnSamePlaneFound = " << edgeAndOptionalLineOnSamePlaneFound << endl;

	return result;
}

bool partOptionalLinesFileOutput(char * partFileModifiedName, char * partFileOriginalName, Reference * initialReferenceInPart)
{
	//int count;

	char c;	//current character being read in
	char * str = " ";

	ifstream parseFileOriginal(partFileOriginalName);
	ofstream partFileModified(partFileModifiedName);

	/*add original part file data*/

  	while ((parseFileOriginal).get(c))
	{
		//count++;
		str[0] = c;
		str[1] = '\0';
		(partFileModified).write(str, 1);
	}
	//cout << "count = " << count << endl;

	/***************************** PART 1 ***********************************************/
	/*determine optional line data (create optional line data reference list)*/
	Reference * initialReferenceInOptionalLineData = new Reference();
	createReferenceListOfOptionalLines(initialReferenceInPart, initialReferenceInOptionalLineData);


	/***************************** PART 2 ***********************************************/
	/*add optional line data to modified part file*/
	char * optionalLineReferenceString = new char[PRIMATIVE_REF_MAX_SIZE];
	Reference * currentReference = initialReferenceInOptionalLineData;
	(partFileModified).write(OPTIONAL_LINE_DATA_HEADER, strlen(OPTIONAL_LINE_DATA_HEADER));
	while(currentReference->next != NULL)		//nothing detected here in 1a3a
	{
		convertReferenceToString(currentReference, optionalLineReferenceString);

		(partFileModified).write(optionalLineReferenceString, strlen(optionalLineReferenceString));

		currentReference = currentReference->next;
	}

	parseFileOriginal.close();
	partFileModified.close();

	return true;
}


/***************************** PART 1 ***********************************************/

void createReferenceListOfOptionalLines(Reference * initialReferenceInPart, Reference * initialReferenceInOptionalLineData)
{
	Reference * currentReferenceInPart = initialReferenceInPart;
	Reference * currentReferenceInOptionalLineData = initialReferenceInOptionalLineData;

	bool currentFunctionResult = false;

	while(currentReferenceInPart->next != NULL)
	{
		if(currentReferenceInPart->type == REFERENCE_TYPE_QUAD)
		{
			numpolysfound++;
			currentReferenceInOptionalLineData = addOptionalLineReferencesForQuad(currentReferenceInPart, currentReferenceInOptionalLineData, initialReferenceInPart, initialReferenceInOptionalLineData, &currentFunctionResult);
		}
		else if(currentReferenceInPart->type == REFERENCE_TYPE_TRI)
		{
			numpolysfound++;
			currentReferenceInOptionalLineData = addOptionalLineReferencesForTri(currentReferenceInPart, currentReferenceInOptionalLineData, initialReferenceInPart, initialReferenceInOptionalLineData, &currentFunctionResult);
		}
		else
		{/*currentReferenceInPart->type == REFERENCE_TYPE_LINE/REFERENCE_TYPE_SUBMODEL/REFERENCE_TYPE_OPTIONALLINE*/
		}

		currentReferenceInPart = currentReferenceInPart->next;
	}
}



//unfinished
Reference * addOptionalLineReferencesForQuad(Reference * quadReference, Reference * currentReferenceInOptionalLineData, Reference * initialReferenceInPart, Reference * initialReferenceInOptionalLineData, bool * result)
{
		//method;

		//calculate all optional line references to be added for the given quad...
			//For every optional line reference calculated, check if optional line reference already exists...
				//if the optional line reference already exists, do not add the optional line reference to the optional line data, currentReferenceInOptionalLineData
				//if the optional line reference does not yet exist, add the optional line reference to the optional line data, currentReferenceInOptionalLineData

		Reference * modCurrentReferenceInOptionalLineData;
		bool currentFunctionResult = false;


		currentReferenceInOptionalLineData = findCorrespondingPrimRefEdgeAndAddOptionalLine(1, &(quadReference->vertex1relativePosition), &(quadReference->vertex2relativePosition), quadReference, currentReferenceInOptionalLineData, initialReferenceInPart, initialReferenceInOptionalLineData, &currentFunctionResult);

		currentReferenceInOptionalLineData = findCorrespondingPrimRefEdgeAndAddOptionalLine(2, &(quadReference->vertex2relativePosition), &(quadReference->vertex3relativePosition), quadReference, currentReferenceInOptionalLineData, initialReferenceInPart, initialReferenceInOptionalLineData, &currentFunctionResult);

		currentReferenceInOptionalLineData = findCorrespondingPrimRefEdgeAndAddOptionalLine(3, &(quadReference->vertex3relativePosition), &(quadReference->vertex4relativePosition), quadReference, currentReferenceInOptionalLineData, initialReferenceInPart, initialReferenceInOptionalLineData, &currentFunctionResult);

		currentReferenceInOptionalLineData = findCorrespondingPrimRefEdgeAndAddOptionalLine(4, &(quadReference->vertex4relativePosition), &(quadReference->vertex1relativePosition), quadReference, currentReferenceInOptionalLineData, initialReferenceInPart, initialReferenceInOptionalLineData, &currentFunctionResult);

		modCurrentReferenceInOptionalLineData = currentReferenceInOptionalLineData;
		*result = currentFunctionResult;
		return modCurrentReferenceInOptionalLineData;
}

//unfinished
Reference * addOptionalLineReferencesForTri(Reference * triReference, Reference * currentReferenceInOptionalLineData, Reference * initialReferenceInPart, Reference * initialReferenceInOptionalLineData, bool * result)
{
		//method;

		//calculate all optional line references to be added for the given quad...
			//For every optional line reference calculated, check if optional line reference already exists...
				//if the optional line reference already exists, do not add the optional line reference to the optional line data, currentReferenceInOptionalLineData
				//if the optional line reference does not yet exist, add the optional line reference to the optional line data, currentReferenceInOptionalLineData

		Reference * modCurrentReferenceInOptionalLineData;
		bool currentFunctionResult = false;

		currentReferenceInOptionalLineData = findCorrespondingPrimRefEdgeAndAddOptionalLine(1, &(triReference->vertex1relativePosition), &(triReference->vertex2relativePosition), triReference, currentReferenceInOptionalLineData, initialReferenceInPart, initialReferenceInOptionalLineData, &currentFunctionResult);

		currentReferenceInOptionalLineData = findCorrespondingPrimRefEdgeAndAddOptionalLine(2, &(triReference->vertex2relativePosition), &(triReference->vertex3relativePosition), triReference, currentReferenceInOptionalLineData, initialReferenceInPart, initialReferenceInOptionalLineData, &currentFunctionResult);

		currentReferenceInOptionalLineData = findCorrespondingPrimRefEdgeAndAddOptionalLine(3, &(triReference->vertex3relativePosition), &(triReference->vertex1relativePosition), triReference, currentReferenceInOptionalLineData, initialReferenceInPart, initialReferenceInOptionalLineData, &currentFunctionResult);

		modCurrentReferenceInOptionalLineData = currentReferenceInOptionalLineData;
		*result = currentFunctionResult;
		return modCurrentReferenceInOptionalLineData;
}



//unfinished
	//edgeNumbers description [polyReferenceEdgeNumber; 1 = vertex 1 and 2, 2 = vertex 2 and 3, 3 = vertex 3 and 4, 4 = vertex 4 and 1)
	/*NB this code assumes that any two coordinates that are equal will always be stated in the same precision in the ldraw part file*/
Reference * findCorrespondingPrimRefEdgeAndAddOptionalLine(int polyReferenceEdgeNumber, vec * polyReferenceVertexA, vec * polyReferenceVertexB, Reference * polyReferenceFull, Reference * currentReferenceInOptionalLineData, Reference * initialReferenceInPart, Reference * initialReferenceInOptionalLineData, bool * result)
{
	Reference * modCurrentReferenceInOptionalLineData;
	bool currentFunctionResult = false;

		//primaryConditions check/////////////////////////////////////////////////////////////
	bool primaryConditions = true;
	Reference lineReference;
	lineReference.type = REFERENCE_TYPE_LINE;
	lineReference.colour = DAT_FILE_DEFAULT_COLOUR_EDGELINE;
	copyVectors(&(lineReference.vertex1relativePosition), polyReferenceVertexA);
	copyVectors(&(lineReference.vertex2relativePosition), polyReferenceVertexB);
	if(search2DRefListFindRef(&lineReference, initialReferenceInPart))
	{
		//do not add an optional line if an ordindary line already exists for the given edge
		primaryConditions = false;
		ordinaryedgesfound++;
		//cout << "here50" << endl;
	}
	copyVectors(&(lineReference.vertex1relativePosition), polyReferenceVertexB);
	copyVectors(&(lineReference.vertex2relativePosition), polyReferenceVertexA);
	if(search2DRefListFindRef(&lineReference, initialReferenceInPart))
	{
		//do not add an optional line if an ordindary line already exists for the given edge
		primaryConditions = false;
		ordinaryedgesfound++;
		//cout << "here50" << endl;
	}
		////////////////////////////////////////////////////////////////////////////////////////

	if(primaryConditions == true)
	{
		Reference * currentReferenceInPart = initialReferenceInPart;

		while(currentReferenceInPart->next != NULL)
		{
				//secondaryConditions check/////////////////////////////////////////////////////////////
			bool secondaryConditions = false;

			if((currentReferenceInPart->type == REFERENCE_TYPE_QUAD) || (currentReferenceInPart->type == REFERENCE_TYPE_TRI))
			{
				secondaryConditions = true;
				if(currentReferenceInPart->type == REFERENCE_TYPE_QUAD)
				{
					if(compareReferences(currentReferenceInPart, polyReferenceFull, REFERENCE_TYPE_QUAD))
					{
						samespolysfound++;
						secondaryConditions = false;
					}
				}
				else if(currentReferenceInPart->type == REFERENCE_TYPE_TRI)
				{
					if(compareReferences(currentReferenceInPart, polyReferenceFull, REFERENCE_TYPE_TRI))
					{
						samespolysfound++;
						secondaryConditions = false;
					}
				}
			}
				////////////////////////////////////////////////////////////////////////////////////////

			if(secondaryConditions == true)
			{
				bool vectorComparison1 = false;
				if(((compareVectors(&(currentReferenceInPart->vertex1relativePosition), polyReferenceVertexA)) && (compareVectors(&(currentReferenceInPart->vertex2relativePosition), polyReferenceVertexB))) || ((compareVectors(&(currentReferenceInPart->vertex1relativePosition), polyReferenceVertexB)) && (compareVectors(&(currentReferenceInPart->vertex2relativePosition), polyReferenceVertexA))))
				{
					vectorComparison1 = true;
				}
				if(vectorComparison1 == true)
				{
					//cout << "vectorComparison1 = true;" << endl;
					//determine optional line required;
					currentReferenceInOptionalLineData = fillOptionalLine(polyReferenceEdgeNumber, polyReferenceVertexA, polyReferenceVertexB, polyReferenceFull, currentReferenceInPart, currentReferenceInOptionalLineData, initialReferenceInOptionalLineData, 1,  &currentFunctionResult);
				}

				bool vectorComparison2 = false;
				if(((compareVectors(&(currentReferenceInPart->vertex2relativePosition), polyReferenceVertexA)) && (compareVectors(&(currentReferenceInPart->vertex3relativePosition), polyReferenceVertexB))) || ((compareVectors(&(currentReferenceInPart->vertex2relativePosition), polyReferenceVertexB)) && (compareVectors(&(currentReferenceInPart->vertex3relativePosition), polyReferenceVertexA))))
				{
					vectorComparison2 = true;
				}
				if(vectorComparison2 == true)
				{
					//cout << "vectorComparison2 = true;" << endl;
					//determine optional line required;
					currentReferenceInOptionalLineData = fillOptionalLine(polyReferenceEdgeNumber, polyReferenceVertexA, polyReferenceVertexB, polyReferenceFull, currentReferenceInPart, currentReferenceInOptionalLineData, initialReferenceInOptionalLineData, 2, &currentFunctionResult);
				}

				bool vectorComparison3 = false;
				if(currentReferenceInPart->type == REFERENCE_TYPE_QUAD)
				{
					if(((compareVectors(&(currentReferenceInPart->vertex3relativePosition), polyReferenceVertexA)) && (compareVectors(&(currentReferenceInPart->vertex4relativePosition), polyReferenceVertexB))) || ((compareVectors(&(currentReferenceInPart->vertex3relativePosition), polyReferenceVertexB)) && (compareVectors(&(currentReferenceInPart->vertex4relativePosition), polyReferenceVertexA))))
					{
						vectorComparison3 = true;

						if(totalNumOptionalLines == 0)
						{
							cout << "currentReferenceInPart->type == REFERENCE_TYPE_QUAD and totalNumOptionalLines == 0 and vectorComparison3 = true" << endl;
						}
					}
				}
				else if(currentReferenceInPart->type == REFERENCE_TYPE_TRI)
				{

					if(((compareVectors(&(currentReferenceInPart->vertex3relativePosition), polyReferenceVertexA)) && (compareVectors(&(currentReferenceInPart->vertex1relativePosition), polyReferenceVertexB))) || ((compareVectors(&(currentReferenceInPart->vertex3relativePosition), polyReferenceVertexB)) && (compareVectors(&(currentReferenceInPart->vertex1relativePosition), polyReferenceVertexA))))
					{
						vectorComparison3 = true;
					}
				}
				if(vectorComparison3 == true)
				{
					//cout << "vectorComparison3 = true;" << endl;
					//determine optional line required;
					currentReferenceInOptionalLineData = fillOptionalLine(polyReferenceEdgeNumber, polyReferenceVertexA, polyReferenceVertexB, polyReferenceFull, currentReferenceInPart, currentReferenceInOptionalLineData, initialReferenceInOptionalLineData, 3, &currentFunctionResult);
				}

				bool vectorComparison4 = false;
				if(currentReferenceInPart->type == REFERENCE_TYPE_QUAD)
				{
					if(((compareVectors(&(currentReferenceInPart->vertex4relativePosition), polyReferenceVertexA)) && (compareVectors(&(currentReferenceInPart->vertex1relativePosition), polyReferenceVertexB))) || ((compareVectors(&(currentReferenceInPart->vertex4relativePosition), polyReferenceVertexB)) && (compareVectors(&(currentReferenceInPart->vertex1relativePosition), polyReferenceVertexA))))
					{
						vectorComparison4 = true;
					}
					if(vectorComparison4 == true)
					{
						//cout << "vectorComparison4 = true;" << endl;
						//determine optional line required;
						currentReferenceInOptionalLineData = fillOptionalLine(polyReferenceEdgeNumber, polyReferenceVertexA, polyReferenceVertexB, polyReferenceFull, currentReferenceInPart, currentReferenceInOptionalLineData, initialReferenceInOptionalLineData, 4, &currentFunctionResult);
					}
				}
			}

			currentReferenceInPart = currentReferenceInPart->next;
		}
	}

	//cout << " numCorrespondingPrimitiveReferenceEdge = " << numCorrespondingPrimitiveReferenceEdge << endl;

	modCurrentReferenceInOptionalLineData = currentReferenceInOptionalLineData;
	*result = currentFunctionResult;
	return modCurrentReferenceInOptionalLineData;

}



Reference * fillOptionalLine(int polyReferenceEdgeNumber, vec * polyReferenceVertexA, vec * polyReferenceVertexB, Reference * polyReferenceFull, Reference * currentReferenceInPart, Reference * currentReferenceInOptionalLineData, Reference * initialReferenceInOptionalLineData, int currentReferenceInPartEdgeNumber, bool * result)
{
	Reference * modCurrentReferenceInOptionalLineData;
	bool currentFunctionResult = false;

	Reference optionalLineReference;
	//determine optional line required;
	optionalLineReference.type = REFERENCE_TYPE_OPTIONALLINE;
	optionalLineReference.colour = DAT_FILE_DEFAULT_COLOUR;
	copyVectors(&(optionalLineReference.vertex1relativePosition), polyReferenceVertexA);
	copyVectors(&(optionalLineReference.vertex2relativePosition), polyReferenceVertexB);
	//add optional specific vertex 1 here now based on the 3D angles of the two adjacent polygons; polyReferenceFull and currentReferenceInPart
	//add optional specific vertex 2 here now based on the 3D angles of the two adjacent polygons; polyReferenceFull and currentReferenceInPart
	if(polyReferenceEdgeNumber == 1)
	{
		if(polyReferenceFull->type == REFERENCE_TYPE_QUAD)
		{
			copyVectors(&(optionalLineReference.vertex3relativePosition), &(polyReferenceFull->vertex3relativePosition));	//or polyReferenceFull->vertex4relativePosition
		}
		else if(polyReferenceFull->type == REFERENCE_TYPE_TRI)
		{
			copyVectors(&(optionalLineReference.vertex3relativePosition), &(polyReferenceFull->vertex3relativePosition));
		}
	}
	else if(polyReferenceEdgeNumber == 2)
	{
		if(polyReferenceFull->type == REFERENCE_TYPE_QUAD)
		{
			copyVectors(&(optionalLineReference.vertex3relativePosition), &(polyReferenceFull->vertex4relativePosition));	//or polyReferenceFull->vertex1relativePosition
		}
		else if(polyReferenceFull->type == REFERENCE_TYPE_TRI)
		{
			copyVectors(&(optionalLineReference.vertex3relativePosition), &(polyReferenceFull->vertex1relativePosition));
		}
	}
	else if(polyReferenceEdgeNumber == 3)
	{
		if(polyReferenceFull->type == REFERENCE_TYPE_QUAD)
		{
			copyVectors(&(optionalLineReference.vertex3relativePosition), &(polyReferenceFull->vertex1relativePosition));	//or polyReferenceFull->vertex2relativePosition
		}
		else if(polyReferenceFull->type == REFERENCE_TYPE_TRI)
		{
			copyVectors(&(optionalLineReference.vertex3relativePosition), &(polyReferenceFull->vertex2relativePosition));
		}
	}
	else if(polyReferenceEdgeNumber == 4)
	{
		if(polyReferenceFull->type == REFERENCE_TYPE_QUAD)
		{
			copyVectors(&(optionalLineReference.vertex3relativePosition), &(polyReferenceFull->vertex2relativePosition));	//or polyReferenceFull->vertex3relativePosition
		}
		else if(polyReferenceFull->type == REFERENCE_TYPE_TRI)
		{
			/*do nothing*/
		}
	}

	if(currentReferenceInPart->type == REFERENCE_TYPE_QUAD)
	{
		if(currentReferenceInPartEdgeNumber == 1)
		{
			copyVectors(&(optionalLineReference.vertex4relativePosition), &(currentReferenceInPart->vertex3relativePosition));	//or currentReferenceInPart->vertex4relativePosition
		}
		else if(currentReferenceInPartEdgeNumber == 2)
		{
			copyVectors(&(optionalLineReference.vertex4relativePosition), &(currentReferenceInPart->vertex4relativePosition));	//or currentReferenceInPart->vertex1relativePosition
		}
		else if(currentReferenceInPartEdgeNumber == 3)
		{
			copyVectors(&(optionalLineReference.vertex4relativePosition), &(currentReferenceInPart->vertex1relativePosition));	//or currentReferenceInPart->vertex2relativePosition
		}
		else if(currentReferenceInPartEdgeNumber == 4)
		{
			copyVectors(&(optionalLineReference.vertex4relativePosition), &(currentReferenceInPart->vertex2relativePosition));	//or currentReferenceInPart->vertex3relativePosition
		}
	}
	else if(currentReferenceInPart->type == REFERENCE_TYPE_TRI)
	{
		if(currentReferenceInPartEdgeNumber == 1)
		{
			copyVectors(&(optionalLineReference.vertex4relativePosition), &(currentReferenceInPart->vertex3relativePosition));
		}
		else if(currentReferenceInPartEdgeNumber == 2)
		{
			copyVectors(&(optionalLineReference.vertex4relativePosition), &(currentReferenceInPart->vertex1relativePosition));
		}
		else if(currentReferenceInPartEdgeNumber == 3)
		{
			copyVectors(&(optionalLineReference.vertex4relativePosition), &(currentReferenceInPart->vertex2relativePosition));
		}
	}


		//tertiaryConditions check/////////////////////////////////////////////////////////////
	bool tertiaryConditions = true;

	if(search2DRefListFindAnyLikeOptionalLineRef(&optionalLineReference, initialReferenceInOptionalLineData))
	{
		//delete optionalLineReference;	//not necessary
		tertiaryConditions = false;
		alreadyaddedoptionallinesfound++;
	}


/*
	Reference optionalLineReferenceRep1;
	Reference optionalLineReferenceRep2;
	Reference optionalLineReferenceRep3;
	Reference optionalLineReferenceRep4;
	vec tempVec;

	copyReferences(&optionalLineReferenceRep1, &optionalLineReference, REFERENCE_TYPE_OPTIONALLINE);
	copyReferences(&optionalLineReferenceRep2, &optionalLineReference, REFERENCE_TYPE_OPTIONALLINE);
	copyReferences(&optionalLineReferenceRep3, &optionalLineReference, REFERENCE_TYPE_OPTIONALLINE);
	copyReferences(&optionalLineReferenceRep4, &optionalLineReference, REFERENCE_TYPE_OPTIONALLINE);

	copyVectors(&tempVec, &(optionalLineReferenceRep2.vertex1relativePosition));
	copyVectors(&(optionalLineReferenceRep2.vertex1relativePosition), &(optionalLineReferenceRep2.vertex2relativePosition));
	copyVectors(&(optionalLineReferenceRep2.vertex2relativePosition), &tempVec);

	copyVectors(&tempVec, &(optionalLineReferenceRep3.vertex3relativePosition));
	copyVectors(&(optionalLineReferenceRep3.vertex3relativePosition), &(optionalLineReferenceRep3.vertex4relativePosition));
	copyVectors(&(optionalLineReferenceRep3.vertex4relativePosition), &tempVec);

	copyVectors(&tempVec, &(optionalLineReferenceRep4.vertex1relativePosition));
	copyVectors(&(optionalLineReferenceRep4.vertex1relativePosition), &(optionalLineReferenceRep4.vertex2relativePosition));
	copyVectors(&(optionalLineReferenceRep4.vertex2relativePosition), &tempVec);
	copyVectors(&tempVec, &(optionalLineReferenceRep4.vertex3relativePosition));
	copyVectors(&(optionalLineReferenceRep4.vertex3relativePosition), &(optionalLineReferenceRep4.vertex4relativePosition));
	copyVectors(&(optionalLineReferenceRep4.vertex4relativePosition), &tempVec);

	if(search2DRefListFindRef(&optionalLineReferenceRep1, initialReferenceInOptionalLineData))
	{
		//delete optionalLineReference;	//not necessary
		tertiaryConditions = false;
	}
	if(search2DRefListFindRef(&optionalLineReferenceRep2, initialReferenceInOptionalLineData))
	{
		//delete optionalLineReference;	//not necessary
		tertiaryConditions = false;
	}
	if(search2DRefListFindRef(&optionalLineReferenceRep3, initialReferenceInOptionalLineData))
	{
		//delete optionalLineReference;	//not necessary
		tertiaryConditions = false;
	}
	if(search2DRefListFindRef(&optionalLineReferenceRep4, initialReferenceInOptionalLineData))
	{
		//delete optionalLineReference;	//not necessary
		tertiaryConditions = false;
	}
	if(tertiaryConditions == false)
	{
		alreadyaddedoptionallinesfound++;
	}
*/



		//quadaryConditions check/////////////////////////////////////////////////////////////
	bool quadaryConditions = true;
#ifndef NO_COLINEAR_OLS_CHECK	/*[make sure these checks are in right/optimum place] */
	/*
	cout << "(optionalLineReference.vertex1relativePosition).x = " << (optionalLineReference.vertex1relativePosition).x << endl;
	cout << "(optionalLineReference.vertex1relativePosition).y = " << (optionalLineReference.vertex1relativePosition).y << endl;
	cout << "(optionalLineReference.vertex2relativePosition).x = " << (optionalLineReference.vertex2relativePosition).x << endl;
	cout << "(optionalLineReference.vertex2relativePosition).y = " << (optionalLineReference.vertex2relativePosition).y << endl;
	*/
	if(edgeLinesVerticiesAndOLVerticsOnSamePlane(&(optionalLineReference.vertex1relativePosition), &(optionalLineReference.vertex2relativePosition), &(optionalLineReference.vertex3relativePosition), &(optionalLineReference.vertex4relativePosition)))
	{
		//delete optionalLineReference;	//not necessary
		quadaryConditions = false;

		bool removeRes = false;
		search2DRefListRemoveAnyLikeOptionalLineRef(&optionalLineReference, initialReferenceInOptionalLineData, &removeRes);

		if(removeRes == true)
		{
			cout << "removed a ref" <<endl;
		}

		edgeAndOptionalLineOnSamePlaneFound++;
	}
#endif

		////////////////////////////////////////////////////////////////////////////////////////

	if((tertiaryConditions == true) && (quadaryConditions == true))
	{
		totalNumOptionalLines++;
		//cout << "totalNumOptionalLines = " << totalNumOptionalLines << endl;

		//copy optionalLineReference into currentReferenceInOptionalLineData
		copyReferences(currentReferenceInOptionalLineData, &optionalLineReference, REFERENCE_TYPE_OPTIONALLINE);

		//template code to create a new reference In Optional Line Data
		Reference * nextReference = new Reference();
		currentReferenceInOptionalLineData -> next = nextReference;
		currentReferenceInOptionalLineData = currentReferenceInOptionalLineData->next;
		currentFunctionResult = true;
	}

	modCurrentReferenceInOptionalLineData = currentReferenceInOptionalLineData;
	*result = currentFunctionResult;
	return modCurrentReferenceInOptionalLineData;
}





Reference * search2DRefListRemoveRef(Reference * referenceToFindAndRemove, Reference * initialReferenceInList, bool * result)
{
	Reference * modInitialReferenceInSceneFile = initialReferenceInList;

	Reference * currentReference = initialReferenceInList;
	Reference * previousReference = NULL;

	while(currentReference->next != NULL)
	{
		bool refFoundInCurrentReference = false;

		if(compareReferences(currentReference, referenceToFindAndRemove, referenceToFindAndRemove->type))
		{
			*result = true;
			refFoundInCurrentReference = true;

			if(previousReference != NULL)
			{
				previousReference->next = currentReference->next;
			}
			else
			{
				modInitialReferenceInSceneFile = currentReference->next;
			}
		}

		previousReference = currentReference;
		currentReference = currentReference->next;

		if(refFoundInCurrentReference == true)
		{
			delete (previousReference);	//remove found unit from reference tree
		}
	}

	return modInitialReferenceInSceneFile;

}

Reference * search2DRefListRemoveAnyLikeOptionalLineRef(Reference * referenceToFindAndRemove, Reference * initialReferenceInList, bool * result)
{
	Reference * modInitialReferenceInSceneFile = initialReferenceInList;

	Reference * currentReference = initialReferenceInList;
	Reference * previousReference = NULL;

	while(currentReference->next != NULL)
	{
		bool refFoundInCurrentReference = false;

		if(compareAnyLikeOptionalLineReferences(currentReference, referenceToFindAndRemove))
		{
			*result = true;
			refFoundInCurrentReference = true;

			if(previousReference != NULL)
			{
				previousReference->next = currentReference->next;
			}
			else
			{
				modInitialReferenceInSceneFile = currentReference->next;
			}
		}

		previousReference = currentReference;
		currentReference = currentReference->next;

		if(refFoundInCurrentReference == true)
		{
			delete (previousReference);	//remove found unit from reference tree
		}
	}

	return modInitialReferenceInSceneFile;

}

bool search2DRefListFindAnyLikeOptionalLineRef(Reference * referenceToFind, Reference * initialReferenceInList)
{
	bool foundReference = false;

	Reference * currentReference = initialReferenceInList;

	while(currentReference->next != NULL)
	{
		if(compareAnyLikeOptionalLineReferences(currentReference, referenceToFind))
		{
			foundReference = true;
		}

		currentReference = currentReference->next;
	}

	return foundReference;
}



bool compareAnyLikeOptionalLineReferences(Reference * reference1, Reference * reference2)
{
	bool result = true;

	if(reference1->type != reference2->type)
	{
		result = false;
	}
	if(reference1->colour != reference2->colour)
	{
		result = false;
	}

	if(!((compareVectors(&(reference1->vertex1relativePosition),  &(reference2->vertex1relativePosition)) && compareVectors(&(reference1->vertex2relativePosition),  &(reference2->vertex2relativePosition))) || (compareVectors(&(reference1->vertex1relativePosition),  &(reference2->vertex2relativePosition)) && compareVectors(&(reference1->vertex2relativePosition),  &(reference2->vertex1relativePosition)))))
	{
		result = false;
	}


	return result;
}







bool edgeLinesVerticiesAndOLVerticsOnSamePlane(vec* edgeLineVertex1, vec* edgeLineVertex2, vec* optionalLineVertex1, vec* optionalLineVertex2)
{
	bool result;

	double xyXInt, zxXInt, xyYInt, yzYInt, yzZInt, zxZInt;

	bool xyinterceptionFound, xyinterceptionPointFound, yzinterceptionFound, yzinterceptionPointFound, zxinterceptionFound, zxinterceptionPointFound;

	xyinterceptionFound = false;
	xyinterceptionPointFound = false;
	yzinterceptionFound = false;
	yzinterceptionPointFound = false;
	zxinterceptionFound = false;
	zxinterceptionPointFound = false;

	/*
	if(totalNumOptionalLines == 12)
	{
		cout << "xy plane" << endl;
	}
	*/
	find2DIntersectionPoint(edgeLineVertex1->x, edgeLineVertex1->y, optionalLineVertex1->x, optionalLineVertex1->y, edgeLineVertex2->x, edgeLineVertex2->y, optionalLineVertex2->x, optionalLineVertex2->y, &xyXInt, &xyYInt, &xyinterceptionFound, &xyinterceptionPointFound);
	//xy plane

	/*
	if(totalNumOptionalLines == 12)
	{
		cout << "yz plane" << endl;
	}
	*/
	find2DIntersectionPoint(edgeLineVertex1->y, edgeLineVertex1->z, optionalLineVertex1->y, optionalLineVertex1->z, edgeLineVertex2->y, edgeLineVertex2->z, optionalLineVertex2->y, optionalLineVertex2->z, &yzYInt, &yzZInt, &yzinterceptionFound, &yzinterceptionPointFound);
	//yz plane

	/*
	if(totalNumOptionalLines == 12)
	{
		cout << "zx plane" << endl;
	}
	*/
	find2DIntersectionPoint(edgeLineVertex1->z, edgeLineVertex1->x, optionalLineVertex1->z, optionalLineVertex1->x, edgeLineVertex2->z, edgeLineVertex2->x, optionalLineVertex2->z, optionalLineVertex2->x, &zxZInt, &zxXInt, &zxinterceptionFound, &zxinterceptionPointFound);
	//zx plane

	/*
	if(totalNumOptionalLines == 12)
	{
		cout << "\nDEBUG: "<< endl;
		cout << "edgeLineVertex1->x = " << edgeLineVertex1->x << endl;
		cout << "edgeLineVertex1->y = " << edgeLineVertex1->y << endl;
		cout << "edgeLineVertex1->z = " << edgeLineVertex1->z << endl;
		cout << "edgeLineVertex2->x = " << edgeLineVertex2->x << endl;
		cout << "edgeLineVertex2->y = " << edgeLineVertex2->y << endl;
		cout << "edgeLineVertex2->z = " << edgeLineVertex2->z << endl;
		cout << "optionalLineVertex1->x = " << optionalLineVertex1->x << endl;
		cout << "optionalLineVertex1->y = " << optionalLineVertex1->y << endl;
		cout << "optionalLineVertex1->z = " << optionalLineVertex1->z << endl;
		cout << "optionalLineVertex2->x = " << optionalLineVertex2->x << endl;
		cout << "optionalLineVertex2->y = " << optionalLineVertex2->y << endl;
		cout << "optionalLineVertex2->z = " << optionalLineVertex2->z << endl;
		cout << "xyinterceptionFound = " << xyinterceptionFound << endl;
		cout << "xyinterceptionPointFound = " << xyinterceptionPointFound << endl;
		cout << "yzinterceptionFound = " << yzinterceptionFound << endl;
		cout << "yzinterceptionPointFound = " << yzinterceptionPointFound << endl;
		cout << "zxinterceptionFound = " << zxinterceptionFound << endl;
		cout << "zxinterceptionPointFound = " << zxinterceptionPointFound << endl;
		cout << "xyXInt = " << xyXInt << endl;
		cout << "zxXInt = " << zxXInt << endl;
		cout << "xyYInt = " << xyYInt << endl;
		cout << "yzYInt = " << yzYInt << endl;
		cout << "yzZInt = " << yzZInt << endl;
		cout << "zxZInt = " << zxZInt << endl;
	}
	*/

	if(xyinterceptionFound && yzinterceptionFound && zxinterceptionFound)
	{
		result = true;

		if(xyinterceptionPointFound && zxinterceptionPointFound)
		{
			if(!compareDoubles(xyXInt, zxXInt))
			{//xy plane
				result = false;
			}
			else
			{
				//cout << "DEBUG: X intersection on xy plane = X intersection on zx plane detected" << endl;
			}
		}
		else
		{
			//cout << "DEBUG: X intersection on xy plane = X intersection on zx plane detected" << endl;
		}

		if(xyinterceptionPointFound && yzinterceptionPointFound)
		{
			if(!compareDoubles(xyYInt, yzYInt))
			{//yz plane
				result = false;
			}
			else
			{
				//cout << "DEBUG: Y intersection on xy plane = Y intersection on yz plane detected" << endl;
			}
		}
		else
		{
			//cout << "DEBUG: Y intersection on xy plane = Y intersection on yz plane detected" << endl;
		}

		if(yzinterceptionPointFound && zxinterceptionPointFound)
		{
			if(!compareDoubles(yzZInt, zxZInt))
			{//zx plane
				result = false;
			}
			else
			{
				//cout << "DEBUG: Z intersection on yz plane = Z intersection on zx plane detected" << endl;
			}
		}
		else
		{
			//cout << "DEBUG: Z intersection on yz plane = Z intersection on zx plane detected" << endl;
		}
	}
	else
	{
		result = false;
	}


	if(result == true)
	{
		/*
		if(totalNumOptionalLines == 12)
		{
			cout << "\nIntersection on all planes detected\n" << endl;
		}
		*/
	}

	//cout << "" << endl;

	return result;
}



bool twoPointsAreTheSame2D(double x1, double y1, double x2, double y2)
{
	bool result;

	if(compareDoubles(calcDistanceBetweenTwoPoints2D(x1, y1, x2, y2), 0.0))
	{
		result = true;
	}
	else
	{
		result = false;
	}

	return result;
}

//void find2DIntersectionPoint(double ax, double ay, double bx, double by, double cx, double cy, double dx, double dy, double * intersectionX, double * intersectionY, bool * interceptionFound, bool * xinterceptionPointFound, bool * yinterceptionPointFound)	//NB an interception may be a line and not a point

//NB uses 0 is the centre of the quad drawn by the 4 points. a, b, c, d.
void find2DIntersectionPoint(double ax, double ay, double bx, double by, double cx, double cy, double dx, double dy, double * intersectionX, double * intersectionY, bool * interceptionFound, bool * interceptionPointFound)	//NB an interception may be a line and not a point
{
	double distanceab;
	double distancebc;
	double distancecd;
	double distanceda;

	double distanceac;
	double distancebd;

	//cout << "DEBUG: here1" << endl;

	distanceab = calcDistanceBetweenTwoPoints2D(ax, ay, bx, by);
	distancebc = calcDistanceBetweenTwoPoints2D(bx, by, cx, cy);
	distancecd = calcDistanceBetweenTwoPoints2D(cx, cy, dx, dy);
	distanceda = calcDistanceBetweenTwoPoints2D(dx, dy, ax, ay);


	distanceac = calcDistanceBetweenTwoPoints2D(ax, ay, cx, cy);
	distancebd = calcDistanceBetweenTwoPoints2D(bx, by, dx, dy);

	bool interceptionPointCannotBeFound = false;

	//safe guard **

	//try for analytical solution

	double mEdge;
	double iEdge;
	double mOptionalLine;
	double iOptionalLine;
	bool edgeGradientInfinityDetected = false;
	bool optionalLineGradientInfinityDetected = false;
	bool edgeNoLineDetected = false;
	bool optionalLineNoLineDetected = false;

	if(!twoPointsAreTheSame2D(ax, ay, cx, cy))
	{
		if(solve2DLineEquationWithTwoPoints(ax, ay, cx, cy, &mEdge, &iEdge))
		{//m != infinity

		}
		else
		{//m = infinity detected, y=iEdge
			edgeGradientInfinityDetected = true;
		}
	}
	else
	{
		edgeNoLineDetected = true;
	}

	if(!twoPointsAreTheSame2D(bx, by, dx, dy))
	{
		if(solve2DLineEquationWithTwoPoints(bx, by, dx, dy, &mOptionalLine, &iOptionalLine))
		{//m != infinity

		}
		else
		{//m = infinity detected, y=iOptionalLine
			optionalLineGradientInfinityDetected = true;
		}
	}
	else
	{
		optionalLineNoLineDetected = true;
	}

	if(!edgeGradientInfinityDetected && !optionalLineGradientInfinityDetected && !edgeNoLineDetected && !optionalLineNoLineDetected)
	{
		if(find2DIntersectionPointOfTwoLines(mEdge, iEdge, mOptionalLine, iOptionalLine, intersectionX, intersectionY))
		{
			*interceptionPointFound = true;

			/*
			if(totalNumOptionalLines == 12)
			{
				cout << "\nDEBUG: "<< endl;
				cout << "totalNumOptionalLines = " << totalNumOptionalLines << endl;
				cout << "\t ax = " << ax << endl;
				cout << "\t ay = " << ay << endl;
				cout << "\t bx = " << bx << endl;
				cout << "\t by = " << by << endl;
				cout << "\t cx = " << cx << endl;
				cout << "\t cy = " << cy << endl;
				cout << "\t dx = " << dx << endl;
				cout << "\t dy = " << dy << endl;
				cout << "\t mEdge = " << mEdge << endl;
				cout << "\t iEdge = " << iEdge << endl;
				cout << "\t mOptionalLine = " << mOptionalLine << endl;
				cout << "\t iOptionalLine = " << iOptionalLine << endl;
				cout << "\t intersectionX = " << *intersectionX << endl;
				cout << "\t intersectionY = " << *intersectionY << endl;
			}
			*/
		}
		else
		{//edge and optional lines are parallel as their gradients are equal, therefore no intersection occurs unless their y intercepts are equal also
			//cout << " here " << endl;

			if(compareDoubles(iEdge, iOptionalLine))
			{
				//return from function that there is no precise interception point on this plane (Eg xy plane).
				*interceptionFound = true;
			}
		}
	}
	else if(!edgeNoLineDetected && !optionalLineNoLineDetected)
	{

		if(edgeGradientInfinityDetected && !optionalLineGradientInfinityDetected)
		{
			*interceptionPointFound = true;

			*intersectionX  = ax;
			*intersectionY = mOptionalLine*ax + iOptionalLine;	//y = mx + i
		}
		else if(!edgeGradientInfinityDetected && optionalLineGradientInfinityDetected)
		{
			*interceptionPointFound = true;

			*intersectionX = bx;
			*intersectionY = mEdge*bx + iEdge;	//y = mx + i
		}
		else if(edgeGradientInfinityDetected && optionalLineGradientInfinityDetected)
		{//edge and optional lines are parallel as their gradients are equal, therefore no intersection occurs unless their x intercepts are equal also
			if(compareDoubles(ax, bx))
			{
				//return from function that there is no precise interception point on this plane (Eg xy plane).
				*interceptionFound = true;
			}
		}

	}
	else
	{
		if((edgeNoLineDetected) && (optionalLineNoLineDetected))
		{//not done yet
			if(twoPointsAreTheSame2D(ax, ay, bx, by))
			{
				*interceptionPointFound = true;
				*intersectionX = ax;
				*intersectionY = ay;
			}
		}
		else if(edgeNoLineDetected)
		{
			if(!optionalLineGradientInfinityDetected)
			{
				if(determineIfPointLiesOnAKnownLine2D(mOptionalLine, iOptionalLine, ax, ay))
				{
					*interceptionPointFound = true;
					*intersectionX = ax;
					*intersectionY = ay;
				}
			}
			else
			{
				if(compareDoubles(ax, bx))
				{
					*interceptionPointFound = true;
					*intersectionX = ax;
					*intersectionY = ay;
				}
			}
		}
		else if(optionalLineNoLineDetected)
		{
			if(!edgeGradientInfinityDetected)
			{
				if(determineIfPointLiesOnAKnownLine2D(mEdge, iEdge, bx, by))
				{
					*interceptionPointFound = true;
					*intersectionX = bx;
					*intersectionY = by;
				}
			}
			else
			{
				if(compareDoubles(bx, ax))
				{
					*interceptionPointFound = true;
					*intersectionX = bx;
					*intersectionY = by;
				}
			}
		}
	}

	if(*interceptionPointFound)
	{
		*interceptionFound = true;
	}
}


bool find2DIntersectionPointOfTwoLines(double m1, double i1, double m2, double i2, double * xIntersection, double * yIntersection)
{
	bool result;

	/*
	/*theory
	m1, m2, i1, i2 are known, find x, and y, in y = m1x + i1, and y = m2x + i2

	y = m1x + i1
	y = m2x + i2
	---

	0 = (m1 - m2)x + (i1 - i2)
	x = -(i1 - i2)/(m1-m2)
	y = m1*x + i1
	*/

	double mNumerator = - (i1-i2);
	double mDenomenator = (m1-m2);

	if(mDenomenator == 0)
	{
		*xIntersection = 0;	//no intersection found
		result = false;
	}
	else
	{
		*xIntersection = mNumerator/mDenomenator;
		result = true;
	}

	*yIntersection = m1*(*xIntersection) + i1;

	return result;
}


double calcDistanceBetweenTwoPoints2D(double x1, double y1, double x2, double y2)
{
	double xDistanceBetweenTheTwoPoints = x1 -  x2;
	double yDistanceBetweenTheTwoPoints = y1 -  y2;
	return sqrt(pow(xDistanceBetweenTheTwoPoints, 2) + pow(yDistanceBetweenTheTwoPoints, 2));
}


bool solve2DLineEquationWithTwoPoints(double x1, double y1, double x2, double y2, double * m, double * i)
{
	bool result;

	/*theory
	find m and i, in y = mx + i

	y1 = mx1 + i
	y2 = mx2 + i
	--
	y1 - y2 = mx1 - mx2
	m = (y1-y2)/(x1-x2)
	---
	i = y1 - ((y1-y2)/(m1-x2) * x1)

	*/

	double mNumerator = (y1-y2);
	double mDenomenator = (x1-x2);
	if(mDenomenator == 0)
	{
		#define VERY_LARGE_DOUBLE 999999999999;
		*m = VERY_LARGE_DOUBLE;
		result = false;
	}
	else
	{
		*m = mNumerator/mDenomenator;
		result = true;
	}
	*i = y1 - ((*m) * x1);
	return result;
}

bool determineIfPointLiesOnLine2D(double x1, double y1, double x2, double y2, double x3, double y3)
{
	bool result = false;

	double m, i;
	if(solve2DLineEquationWithTwoPoints(x1, y1, x2, y2, &m, &i))
	{//m != infinity
		double RHS = m*(x3) + i;
		if(compareDoubles(y3, RHS))
		{
			result = true;
		}
	}
	else
	{//m = infinity detected
		if(compareDoubles(x1, x3))
		{
			result = true;
		}
	}

	return result;
}


bool determineIfPointLiesOnAKnownLine2D(double m1, double i1, double x3, double y3)
{//preconditions: m != infinity
	bool result = false;


	double RHS = m1*(x3) + i1;
	if(compareDoubles(y3, RHS))
	{
		result = true;
	}

	return result;
}


